"""A terminal-based Markdown document viewer, written in Textual."""

__author__ = "Textualize, Inc"
__copyright__ = "Copyright Textualize, Inc"
__credits__ = ["Dave Pearson"]
__maintainer__ = "Dave Pearson"
__email__ = "dave@textualize.io"
__version__ = "0.6.0"
__licence__ = "MIT"
